@@ -4,6 +4,7 @@
 """ 
 Collection of functions to convert audio and voltage data to Sound Pressure 
 Level (SPL in Pascal) and Leq (Continuous Equivalent SPL).
+Functions are adapted from the algorithm proposed in Merchant, N.D., Fristrup, K.M., Johnson, M.P., Tyack, P.L., Witt, M.J., Blondel, P. and Parks, S.E. (2015), Measuring acoustic habitats. Methods Ecol Evol, 6: 257-265. `DOI: 10.1111/2041-210X.12330 <https://doi.org/10.1111/2041-210X.12330>`_
 """   
 #
 # Authors:  Juan Sebastian ULLOA <lisofomia@gmail.com>
@@ -522,7 +523,7 @@ def pressure2leq (p, fs, dt=1, pRef = 20e-6):
     # reshape into 2D (the duration of each row is dt)
     p_2D = p_1D.reshape(N_RMS,dN)
     # RMS
-    p_RMS = sqrt(mean(p_2D**2,axis=1))# Test the current operating system
+    p_RMS = sqrt(mean(p_2D**2,axis=1))
     # if p_RMS ==0 set to MIN
     p_RMS[p_RMS==0] = _MIN_
     # RMS to Leq (Equivalent Continuous Sound level)
